@@ -56,14 +56,17 @@
  *	v1.43 (pc) - Added support for Cintiq 21UX
  *		   - Fixed a Graphire bug
  *		   - Merged wacom_intuos3_irq into wacom_intuos_irq
- *	v1.44 (pc) - Added support for Graphire4, Cintiq 710, Intuos3 6x11, etc.
- *		   - Report Device IDs
- *      v1.45 (pc) - Added support for DTF 521, Intuos3 12x12 and 12x19
- *                 - Minor data report fix
- *      v1.46 (pc) - Split wacom.c into wacom_sys.c and wacom_wac.c, 
- *		   - where wacom_sys.c deals with system specific code, 
- * 		   - and wacom_wac.c deals with Wacom specific code
- *		   - Support Intuos3 4x6
+ *
+ *
+ *   WARNING: THIS IS NOT PART OF THE OFFICIAL KERNEL TREE
+ *   THIS IS FOR TESTING PURPOSES
+ *
+ *    v1.43-2.6.13.1-pc-0.1 - initial release based on 2.6.13.1
+ *    v1.43-2.6.13.1-pc-0.2 - added DTF 521, I3 12x12, and I3 12x19
+ *    v1.43-2.6.13.1-pc-0.3 - Support tablet buttons/keys
+ *    v1.43-2.6.13.1-pc-0.4 - Split wacom.c into 4 files
+ *    v1.43-2.6.13.1-pc-0.5 - added Bamboo
+ *    v1.43-2.6.13.1-pc-0.6 - Added BambooFun and Hummingbird
  */
 
 /*
@@ -76,15 +79,18 @@
 #define WACOM_H
 #include <linux/kernel.h>
 #include <linux/slab.h>
+#include <linux/input.h>
 #include <linux/module.h>
 #include <linux/init.h>
-#include <linux/usb/input.h>
+#include <linux/usb.h>
+#include <linux/usb_input.h>
 #include <asm/unaligned.h>
+#include <asm/byteorder.h>
 
 /*
  * Version Information
  */
-#define DRIVER_VERSION "v1.46-pc0.2"
+#define DRIVER_VERSION "v1.43-2.6.13.1-pc-0.6"
 #define DRIVER_AUTHOR "Vojtech Pavlik <vojtech@ucw.cz>"
 #define DRIVER_DESC "USB Wacom Graphire and Wacom Intuos tablet driver"
 #define DRIVER_LICENSE "GPL"
@@ -97,10 +103,11 @@ MODULE_LICENSE(DRIVER_LICENSE);
 
 struct wacom {
 	dma_addr_t data_dma;
-	struct input_dev *dev;
+	struct input_dev dev;
 	struct usb_device *usbdev;
 	struct urb *irq;
 	struct wacom_wac * wacom_wac;
+	int open;
 	char phys[32];
 };
 
