@@ -2,7 +2,7 @@
 ** wacusb.c
 **
 ** Copyright (C) 2002 - 2004 - John E. Joganic
-** Copyright (C) 2003 - 2007 - Ping Cheng
+** Copyright (C) 2003 - 2008 - Ping Cheng
 **
 ** This program is free software; you can redistribute it and/or
 ** modify it under the terms of the GNU General Public License
@@ -20,6 +20,8 @@
 **
 ****************************************************************************/
 
+#include "../include/util-config.h"
+
 #include "wacusb.h"
 
 #include <stdio.h>
@@ -31,10 +33,6 @@
 #include <sys/ioctl.h>
 #include <assert.h>
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
 /*****************************************************************************
 ** Begin USB Linux Input Subsystem
 *****************************************************************************/
@@ -253,12 +251,15 @@ static int USBIdentifyModel(USBTABLET* pUSB);
 		{ "CTF-420-U", "Wacom Volito2 4x5", 2, 0x62 },
 		{ "CTF-220-U", "Wacom Volito2 2x3", 3, 0x63 },
 		{ "CTF-421-U", "Wacom PenPartner2", 4, 0x64 },
+		{ "CTF_430-U", "Wacom Bamboo1",     5, 0x69 },
 		{ NULL }
 	};
 
 	static USBSUBTYPE xBamboo[] =
 	{
 		{ "MTE_450", "Wacom Bamboo", 1, 0x65 },
+		{ "CTE_450", "Wacom BambooFun 4x5", 2, 0x17 },
+		{ "CTE_650", "Wacom BambooFun 6x8", 3, 0x18 },
 		{ NULL }
 	};
 
@@ -270,7 +271,9 @@ static int USBIdentifyModel(USBTABLET* pUSB);
 
 	static USBSUBTYPE xCintiqV5[] =
 	{
-		{ "DTZ-21ux", "Wacom Cintiq 21UX", 1, 0x3F },
+		{ "DTZ-21ux",  "Wacom Cintiq 21UX",  1, 0x3F },
+		{ "DTZ-20wsx", "Wacom Cintiq 20WSX", 2, 0x65 },
+		{ "DTZ-12wx",  "Wacom Cintiq 12WX",  3, 0x6C },
 		{ NULL }
 	};
 
