@@ -1,7 +1,7 @@
 /*****************************************************************************
 ** wcmAction.c
 **
-** Copyright (C) 2007 - Ping Cheng
+** Copyright (C) 2007 - 2008 - Ping Cheng
 **
 ** This program is free software; you can redistribute it and/or
 ** modify it under the terms of the GNU General Public License
@@ -19,6 +19,7 @@
 **
 ** REVISION HISTORY
 **   2007-02-06 0.0.1-pc - Support keystrokes
+**   2008-01-17 0.0.2-pc - Add Display Toggle
 */
 
 /* This pseudo-header file is included both from the X11 driver, and from
@@ -41,7 +42,8 @@ static ACTIONCODE action_code [] =
 	{ "KEY",        AC_KEY },
 	{ "BUTTON",     AC_BUTTON },
 	{ "MODETOGGLE", AC_MODETOGGLE },
-	{ "DBLCLICK",   AC_DBLCLICK }
+	{ "DBLCLICK",   AC_DBLCLICK },
+	{ "DISPLAYTOGGLE", AC_DISPLAYTOGGLE }
 };
 
 static ACTIONCODE modifier_code [] =
@@ -238,6 +240,7 @@ int xf86WcmDecode (const char *dev, const char *but, const char *ev, unsigned *
 	    }
 	case AC_MODETOGGLE:
 	case AC_DBLCLICK:
+	case AC_DISPLAYTOGGLE:
 	    break;
 	case AC_KEY:
 	    if (!strlen(ev_p))
