@@ -1,6 +1,6 @@
 /*
  * Copyright 1995-2002 by Frederic Lepied, France. <Lepied@XFree86.org>
- * Copyright 2002-2007 by Ping Cheng, Wacom Technology. <pingc@wacom.com>		
+ * Copyright 2002-2008 by Ping Cheng, Wacom Technology. <pingc@wacom.com>		
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -21,7 +21,7 @@
 #include "../include/Xwacom.h"
 
 /*
- #if XF86_VERSION_MAJOR < 4
+#if XF86_VERSION_MAJOR < 4
  *
  * There is a bug in XFree86 for combined left click and
  * other button. It'll lost left up when releases.
@@ -34,7 +34,7 @@
 
 WacomDeviceClass* wcmDeviceClasses[] =
 {
-#ifdef LINUX_INPUT
+#ifdef WCM_ENABLE_LINUXINPUT
 	&gWacomUSBDevice,
 #endif
 	&gWacomISDV4Device,
@@ -42,6 +42,9 @@ WacomDeviceClass* wcmDeviceClasses[] =
 	NULL
 };
 
+extern int xf86WcmDevSwitchModeCall(LocalDevicePtr local, int mode);
+extern void xf86WcmChangeScreen(LocalDevicePtr local, int value);
+
 /*****************************************************************************
  * Static functions
  ****************************************************************************/
@@ -53,6 +56,98 @@ static void resetSampleCounter(const WacomChannelPtr pChannel);
 static void sendAButton(LocalDevicePtr local, int button, int mask,
 		int rx, int ry, int rz, int v3, int v4, int v5);
  
+/*****************************************************************************
+ * xf86WcmMappingFactor --
+ *   calculate the proper tablet to screen mapping factor according to the 
+ *   screen/desktop size and the tablet size 
+ ****************************************************************************/
+
+void xf86WcmMappingFactor(LocalDevicePtr local)
+{
+	WacomDevicePtr priv = (WacomDevicePtr) local->private;
+	int i = 0, minX = 0, minY = 0, maxX = 0, maxY = 0;
+ 
+	DBG(10, priv->debugLevel, ErrorF("xf86WcmMappingFactor \n"));
+
+	priv->sizeX = priv->bottomX - priv->topX - 2*priv->tvoffsetX;
+	priv->sizeY = priv->bottomY - priv->topY - 2*priv->tvoffsetY;
+	priv->maxWidth = 0, priv->maxHeight = 0;
+	
+	if (!(priv->flags & ABSOLUTE_FLAG) || !priv->wcmMMonitor)
+	{
+		/* Get the current screen that the cursor is in */
+#if defined WCM_XFREE86 || GET_ABI_MAJOR(ABI_XINPUT_VERSION) == 0
+		if (miPointerCurrentScreen())
+			priv->currentScreen = miPointerCurrentScreen()->myNum;
+#else
+		if (miPointerGetScreen(local->dev))
+			priv->currentScreen = miPointerGetScreen(local->dev)->myNum;
+#endif
+	}
+	else
+	{
+		if (priv->screen_no != -1)
+			priv->currentScreen = priv->screen_no;
+		else if (priv->currentScreen == -1)
+		{
+			/* Get the current screen that the cursor is in */
+#if defined WCM_XFREE86 || GET_ABI_MAJOR(ABI_XINPUT_VERSION) == 0
+			if (miPointerCurrentScreen())
+				priv->currentScreen = miPointerCurrentScreen()->myNum;
+#else
+			if (miPointerGetScreen(local->dev))
+				priv->currentScreen = miPointerGetScreen(local->dev)->myNum;
+#endif
+		}
+	}
+	if (priv->currentScreen == -1) /* tool on the tablet */
+		priv->currentScreen = 0;
+
+	if ( ((priv->twinview != TV_NONE) || /* TwinView & whole desktop */
+		/* stay in one screen at a time (multimonitor) */
+		!priv->wcmMMonitor || 
+		/* always stay in the configured screen */
+		(screenInfo.numScreens > 1 && priv->screen_no != -1))  
+		 && (priv->flags & ABSOLUTE_FLAG) )
+	{
+		priv->maxWidth = priv->screenBottomX[priv->currentScreen] - 
+			priv->screenTopX[priv->currentScreen];
+		priv->maxHeight = priv->screenBottomY[priv->currentScreen] - 
+			priv->screenTopY[priv->currentScreen];
+	}
+	else
+	{
+		/* count the whole desktop when no specific screen is defined or 
+		 * tool is in relative mode
+		 */
+		minX = priv->screenTopX[0];
+		minY = priv->screenTopY[0];
+		maxX = priv->screenBottomX[0];
+		maxY = priv->screenBottomY[0];
+		for (i = 1; i < priv->numScreen; i++)
+		{
+			if (priv->screenTopX[i] < minX)
+				minX = priv->screenTopX[i];
+			if (priv->screenTopY[i] < minY)
+				minY = priv->screenTopY[i];
+			if (priv->screenBottomX[i] > maxX)
+				maxX = priv->screenBottomX[i];
+			if (priv->screenBottomY[i] > maxY)
+				maxY = priv->screenBottomY[i];
+		}
+		priv->maxWidth = maxX - minX;
+		priv->maxHeight = maxY - minY;
+	}
+	DBG(10, priv->debugLevel, ErrorF("xf86WcmMappingFactor"
+		" Active tablet area x=%d y=%d map to maxWidth =%d maxHeight =%d\n", 
+		priv->sizeX, priv->sizeY, priv->maxWidth, priv->maxHeight));
+
+	priv->factorX = (double)priv->maxWidth / (double)priv->sizeX;
+	priv->factorY = (double)priv->maxHeight / (double)priv->sizeY;
+	DBG(2, priv->debugLevel, ErrorF("X factor = %.3g, Y factor = %.3g\n",
+		priv->factorX, priv->factorY));
+}
+
 /*****************************************************************************
  * xf86WcmSetScreen --
  *   set to the proper screen according to the converted (x,y).
@@ -64,118 +159,83 @@ static void sendAButton(LocalDevicePtr local, int button, int mask,
 static void xf86WcmSetScreen(LocalDevicePtr local, int *value0, int *value1)
 {
 	WacomDevicePtr priv = (WacomDevicePtr) local->private;
-	int screenToSet = 0;
-	int totalWidth = 0, maxHeight = 0, leftPadding = 0;
-	int i, x, y, v0 = *value0, v1 = *value1;
-	double sizeX = priv->bottomX - priv->topX - 2*priv->tvoffsetX;
-	double sizeY = priv->bottomY - priv->topY - 2*priv->tvoffsetY;
+	int screenToSet = -1, letfPadding = 0, topPadding = 0;
+	int i, j, x, y, v0 = *value0, v1 = *value1;
 
 	DBG(6, priv->debugLevel, ErrorF("xf86WcmSetScreen "
-		"v0=%d v1=%d\n", *value0, *value1));
+		"v0=%d v1=%d currentScreen=%d\n", *value0, 
+		*value1, priv->currentScreen));
 
 	if (!(local->flags & (XI86_ALWAYS_CORE | XI86_CORE_POINTER))) return;
 
-	if (!(priv->flags & ABSOLUTE_FLAG))
+	if (priv->twinview != TV_NONE && priv->screen_no == -1 && (priv->flags & ABSOLUTE_FLAG))
 	{
-#if defined WCM_XFREE86 || GET_ABI_MAJOR(ABI_XINPUT_VERSION) == 0
-		priv->currentScreen = miPointerCurrentScreen()->myNum;
-#else
-		priv->currentScreen = miPointerGetScreen(local->dev)->myNum;
-#endif
-		for (i = 0; i < priv->numScreen; i++)
-			totalWidth += screenInfo.screens[i]->width;
-
-		maxHeight = screenInfo.screens[priv->currentScreen]->height;
-		priv->factorX = totalWidth / sizeX;
-		priv->factorY = maxHeight / sizeY;
-		DBG(10, priv->debugLevel, ErrorF(
-			"xf86WcmSetScreen current=%d ToSet=%d\n", 
-			priv->currentScreen, screenToSet));
-		return;
-	}
-
-	if (priv->twinview == TV_NONE)
-	{
-		v0 = v0 > priv->bottomX ? priv->bottomX - priv->topX :
-			v0 < priv->topX ? 0 : v0 - priv->topX;
-		v1 = v1 > priv->bottomY ? priv->bottomY - priv->topY :
-			v1 < priv->topY ? 0 : v1 - priv->topY;
-	}
-
-	/* set factorX and factorY for single screen setup since
-	 * Top X Y and Bottom X Y can be changed while driver is running
-	 */
-	if (screenInfo.numScreens == 1 || !priv->common->wcmMMonitor)
-	{
-		if (priv->twinview != TV_NONE)
+		if (priv->twinview == TV_LEFT_RIGHT)
 		{
-			if (priv->screen_no == -1)
-			{
-				if (priv->twinview == TV_LEFT_RIGHT)
-				{
-					if (v0 > priv->bottomX - priv->tvoffsetX && v0 <= priv->bottomX)
-						priv->currentScreen = 1;
-					if (v0 > priv->topX && v0 <= priv->topX + priv->tvoffsetX)
-						priv->currentScreen = 0;
-				}
-				if (priv->twinview == TV_ABOVE_BELOW)
-				{
-					if (v1 > priv->bottomY - priv->tvoffsetY && v1 <= priv->bottomY)
-						priv->currentScreen = 1;
-					if (v1 > priv->topY && v1 <= priv->topY + priv->tvoffsetY)
-						priv->currentScreen = 0;
-				}
-			}
-			else
-				priv->currentScreen = priv->screen_no;
-			priv->factorX = priv->tvResolution[2*priv->currentScreen] / sizeX;
-			priv->factorY = priv->tvResolution[2*priv->currentScreen+1] / sizeY;
+			if (v0 > priv->bottomX - priv->tvoffsetX && v0 <= priv->bottomX)
+				priv->currentScreen = 1;
+			if (v0 > priv->topX && v0 <= priv->topX + priv->tvoffsetX)
+				priv->currentScreen = 0;
 		}
-		else
+		if (priv->twinview == TV_ABOVE_BELOW)
 		{
-			/* tool on the tablet when driver starts */
-#if defined WCM_XFREE86 || GET_ABI_MAJOR(ABI_XINPUT_VERSION) == 0
-			if (miPointerCurrentScreen())
-				priv->currentScreen = miPointerCurrentScreen()->myNum;
-#else
-			if (miPointerGetScreen(local->dev))
-				priv->currentScreen = miPointerGetScreen(local->dev)->myNum;
-#endif
-			priv->factorX = screenInfo.screens[priv->currentScreen]->width / sizeX;
-			priv->factorY = screenInfo.screens[priv->currentScreen]->height / sizeY;
+			if (v1 > priv->bottomY - priv->tvoffsetY && v1 <= priv->bottomY)
+				priv->currentScreen = 1;
+			if (v1 > priv->topY && v1 <= priv->topY + priv->tvoffsetY)
+				priv->currentScreen = 0;
 		}
-		return;
 	}
 
+	xf86WcmMappingFactor(local);
+	if (!(priv->flags & ABSOLUTE_FLAG) || screenInfo.numScreens == 1 || !priv->wcmMMonitor)
+		return;
+
+	v0 = v0 - priv->topX;
+	v1 = v1 - priv->topY;
+
 	if (priv->screen_no == -1)
 	{
 		for (i = 0; i < priv->numScreen; i++)
 		{
-			totalWidth += screenInfo.screens[i]->width;
-			if (maxHeight < screenInfo.screens[i]->height)
-				maxHeight = screenInfo.screens[i]->height;
-		}
-		for (i = 0; i < priv->numScreen; i++)
-		{
-			if (v0 * totalWidth <= (leftPadding + 
-				screenInfo.screens[i]->width) * sizeX)
+			if (v0 * priv->factorX >= priv->screenTopX[i] && 
+				v0 * priv->factorX < priv->screenBottomX[i] - 0.5)
 			{
-				screenToSet = i;
-				break;
+				
+				for (j = 0; j < priv->numScreen; j++)
+				{
+					if (v1 * priv->factorY >= priv->screenTopY[j] && 
+						v1 * priv->factorY <= priv->screenBottomY[j] - 0.5)
+					{
+						if (j == i)
+						{
+							screenToSet = i;
+							break;
+						}
+					}
+				}
+					
+				if (screenToSet != -1)
+					break;
 			}
-			leftPadding += screenInfo.screens[i]->width;
+		}
+		if (screenToSet != -1)
+		{
+			letfPadding = priv->screenTopX[screenToSet];
+			topPadding = priv->screenBottomX[screenToSet];
+		}
+		else
+		{
+			DBG(3, priv->debugLevel, ErrorF("xf86WcmSetScreen Error: "
+				"Can not find valid screen (currentScreen=%d)\n", 
+				priv->currentScreen));
+			return;
 		}
 	}
-	else 
-	{
+	else
 		screenToSet = priv->screen_no;
-		totalWidth = screenInfo.screens[screenToSet]->width;
-		maxHeight = screenInfo.screens[screenToSet]->height;
-	}
-	priv->factorX = totalWidth/sizeX;
-	priv->factorY = maxHeight/sizeY;
-	x = (v0 - sizeX * leftPadding / totalWidth) * priv->factorX + 0.5;
-	y = v1 * priv->factorY + 0.5;
+
+	x = (double)v0 * priv->factorX - letfPadding + 0.5;
+	y = (double)v1 * priv->factorY - topPadding + 0.5;
 		
 	if (x >= screenInfo.screens[screenToSet]->width)
 		x = screenInfo.screens[screenToSet]->width - 1;
@@ -184,8 +244,8 @@ static void xf86WcmSetScreen(LocalDevicePtr local, int *value0, int *value1)
 
 	xf86XInputSetScreen(local, screenToSet, x, y);
 	DBG(10, priv->debugLevel, ErrorF("xf86WcmSetScreen"
-		" current=%d ToSet=%d\n", 
-		priv->currentScreen, screenToSet));
+			" current=%d ToSet=%d\n", 
+			priv->currentScreen, screenToSet));
 	priv->currentScreen = screenToSet;
 }
 
@@ -497,18 +557,39 @@ static void sendAButton(LocalDevicePtr local, int button, int mask,
 		break;
 
 	case AC_MODETOGGLE:
-		if (!mask)
-			break;
-
-		if (priv->flags & ABSOLUTE_FLAG)
+		if (mask)
 		{
-			priv->flags &= ~ABSOLUTE_FLAG;
-			xf86ReplaceStrOption(local->options, "Mode", "Relative");
+			int mode = Absolute;
+			if (is_absolute)
+				mode = Relative;
+			xf86WcmDevSwitchModeCall(local, mode);
 		}
-		else
+		break;
+
+	case AC_DISPLAYTOGGLE:
+		if (mask && priv->numScreen > 1)
 		{
-			priv->flags |= ABSOLUTE_FLAG;
-			xf86ReplaceStrOption(local->options, "Mode", "Absolute");
+			if (IsPad(priv)) /* toggle display for all tools except pad */
+			{
+				WacomDevicePtr tmppriv;
+				for (tmppriv = common->wcmDevices; tmppriv; tmppriv = tmppriv->next)
+				{
+					if (!IsPad(tmppriv))
+					{
+						int screen = tmppriv->screen_no;
+						if (++screen >= tmppriv->numScreen)
+							screen = -1;
+						xf86WcmChangeScreen(tmppriv->local, screen);
+					}
+				}
+			}
+			else /* toggle display only for the selected tool */
+			{
+				int screen = priv->screen_no;
+				if (++screen >= priv->numScreen)
+					screen = -1;
+				xf86WcmChangeScreen(local, screen);
+			}
 		}
 		break;
 
@@ -838,14 +919,6 @@ void xf86WcmSendEvents(LocalDevicePtr local, const WacomDeviceState* ds)
 		x, y, z, v3, v4, v5, id, serial,
 		is_button ? "true" : "false", buttons));
 
-	if (x > priv->bottomX)
-		x = priv->bottomX;
-	if (x < priv->topX)
-		x = priv->topX;
-	if (y > priv->bottomY)
-		y = priv->bottomY;
-	if (y < priv->topY)
-		y = priv->topY;
 	priv->currentX = x;
 	priv->currentY = y;
 
@@ -897,13 +970,6 @@ void xf86WcmSendEvents(LocalDevicePtr local, const WacomDeviceState* ds)
 		}		
 	}
 
-#if defined WCM_XORG && GET_ABI_MAJOR(ABI_XINPUT_VERSION) > 0
-	/* Ugly hack for Xorg 7.3, which doesn't call xf86WcmDevConvert
-	 * for coordinate conversion at the moment */
-	x = (int)((double)(x - priv->topX) * priv->factorX + 0.5);
-	y = (int)((double)(y - priv->topY) * priv->factorY + 0.5);
-#endif
-
 	if (type != PAD_ID)
 	{
 		/* coordinates are ready we can send events */
@@ -922,6 +988,22 @@ void xf86WcmSendEvents(LocalDevicePtr local, const WacomDeviceState* ds)
 			if (!is_absolute)
 				x *= priv->factorY / priv->factorX;
 
+#ifdef WCM_XORG_TABLET_SCALING
+			/* Ugly hack for Xorg 7.3, which doesn't call xf86WcmDevConvert
+			 * for coordinate conversion at the moment */
+			/* The +-0.4 is to increase the sensitivity in relative mode.
+			 * Must be sensitive to which way the tool is moved or one way
+			 * will get a severe penalty for small movements. */
+			x = (int)((double)(x - priv->topX) * priv->factorX + (x>=0?0.4:-0.4));
+			y = (int)((double)(y - priv->topY) * priv->factorY + (y>=0?0.4:-0.4));
+
+			/* map to a specific screen */
+			if (priv->screen_no != -1 || priv->twinview != TV_NONE)
+			{
+				x += priv->screenTopX[priv->currentScreen];
+				y += priv->screenTopY[priv->currentScreen];
+			}
+#endif
 			sendCommonEvents(local, ds, x, y, z, v3, v4, v5);
 
 			if(!(priv->flags & BUTTONS_ONLY_FLAG))
@@ -1002,6 +1084,7 @@ void xf86WcmSendEvents(LocalDevicePtr local, const WacomDeviceState* ds)
 		priv->oldStripY = 0;
 		priv->oldRot = 0;
 		priv->oldThrottle = 0;
+		priv->devReverseCount = 0;
 	}
 }
 
@@ -1059,10 +1142,12 @@ Bool xf86WcmOpen(LocalDevicePtr local)
 	WacomDevicePtr priv = (WacomDevicePtr)local->private;
 	WacomCommonPtr common = priv->common;
 	WacomDeviceClass** ppDevCls;
+	char id[BUFFER_SIZE];
+	float version;
 
 	DBG(1, priv->debugLevel, ErrorF("opening %s\n", common->wcmDevice));
 
-	local->fd = xf86WcmOpenTablet(local);
+	local->fd = xf86OpenSerial(local->options);
 	if (local->fd < 0)
 	{
 		ErrorF("Error opening %s : %s\n", common->wcmDevice,
@@ -1081,7 +1166,14 @@ Bool xf86WcmOpen(LocalDevicePtr local)
 	}
 
 	/* Initialize the tablet */
-	return common->wcmDevCls->Init(local);
+	if(common->wcmDevCls->Init(local, id, &version) != Success ||
+		xf86WcmInitTablet(local, id, version) != Success)
+	{
+		xf86CloseSerial(local->fd);
+		local->fd = -1;
+		return !Success;
+	}
+	return Success;
 }
 
 /* reset raw data counters for filters */
@@ -1140,7 +1232,7 @@ void xf86WcmEvent(WacomCommonPtr common, unsigned int channel,
 		ds.discard_first, ds.proximity, ds.sample,
 		pChannel->nSamples));
 
-#ifdef LINUX_INPUT
+#ifdef WCM_ENABLE_LINUXINPUT
 	/* Discard the first 2 USB packages due to events delay */
 	if ( (pChannel->nSamples < 2) && (common->wcmDevCls == &gWacomUSBDevice) && ds.device_type != PAD_ID )
 	{
@@ -1549,13 +1641,13 @@ static void commonDispatchDevice(WacomCommonPtr common, unsigned int channel,
  * xf86WcmInitTablet -- common initialization for all tablets
  ****************************************************************************/
 
-int xf86WcmInitTablet(LocalDevicePtr local, WacomModelPtr model,
-	const char* id, float version)
+int xf86WcmInitTablet(LocalDevicePtr local, const char* id, float version)
 {
-	WacomCommonPtr common =	((WacomDevicePtr)(local->private))->common;
+	WacomCommonPtr common = ((WacomDevicePtr)(local->private))->common;
 	WacomToolPtr toollist = common->wcmTool;
 	WacomToolAreaPtr arealist;
 	int temp;
+	WacomModelPtr model = common->wcmModel;
 
 	/* Initialize the tablet */
 	model->Initialize(common,id,version);
@@ -1647,9 +1739,6 @@ int xf86WcmInitTablet(LocalDevicePtr local, WacomModelPtr model,
 	if (model->Start && (model->Start(local) != Success))
 		return !Success;
 
-	/*set the model */
-	common->wcmModel = model;
-
 	return Success;
 }
 
@@ -1678,3 +1767,136 @@ static void transPressureCurve(WacomDevicePtr pDev, WacomDeviceStatePtr pState)
 			FILTER_PRESSURE_RES;
 	}
 }
+
+/*****************************************************************************
+ * xf86WcmInitialTVScreens
+ ****************************************************************************/
+
+static void xf86WcmInitialTVScreens(LocalDevicePtr local)
+{
+	WacomDevicePtr priv = (WacomDevicePtr)local->private;
+
+	if (priv->twinview == TV_NONE)
+		return;
+
+	priv->numScreen = 2;
+
+	if (priv->twinview == TV_LEFT_RIGHT)
+	{
+		/* it does not need the offset if always map to a specific screen */
+		if (priv->screen_no == -1)
+		{
+			priv->tvoffsetX = 60;
+			priv->tvoffsetY = 0;
+		}
+
+		/* default resolution */
+		if(!priv->tvResolution[0])
+		{
+			priv->tvResolution[0] = screenInfo.screens[0]->width/2;
+			priv->tvResolution[1] = screenInfo.screens[0]->height;
+			priv->tvResolution[2] = priv->tvResolution[0];
+			priv->tvResolution[3] = priv->tvResolution[1];
+		}
+	}
+	else if (priv->twinview == TV_ABOVE_BELOW)
+	{
+		/* it does not need the offset if always map to a specific screen */
+		if (priv->screen_no == -1)
+		{
+			priv->tvoffsetX = 0;
+			priv->tvoffsetY = 60;
+		}
+
+		/* default resolution */
+		if(!priv->tvResolution[0])
+		{
+			priv->tvResolution[0] = screenInfo.screens[0]->width;
+			priv->tvResolution[1] = screenInfo.screens[0]->height/2;
+			priv->tvResolution[2] = priv->tvResolution[0];
+			priv->tvResolution[3] = priv->tvResolution[1];
+		}
+	}
+
+	/* initial screen info */
+	priv->screenTopX[0] = 0;
+	priv->screenTopY[0] = 0;
+	priv->screenBottomX[0] = priv->tvResolution[0];
+	priv->screenBottomY[0] = priv->tvResolution[1];
+	if (priv->twinview == TV_ABOVE_BELOW)
+	{
+		priv->screenTopX[1] = 0;
+		priv->screenTopY[1] = priv->tvResolution[1];
+		priv->screenBottomX[1] = priv->tvResolution[2];
+		priv->screenBottomY[1] = priv->tvResolution[1] + priv->tvResolution[3];
+	}
+	if (priv->twinview == TV_LEFT_RIGHT)
+	{
+		priv->screenTopX[1] = priv->tvResolution[0];
+		priv->screenTopY[1] = 0;
+		priv->screenBottomX[1] = priv->tvResolution[0] + priv->tvResolution[2];
+		priv->screenBottomY[1] = priv->tvResolution[3];
+	}
+
+	DBG(10, priv->debugLevel, ErrorF("xf86WcmInitialTVScreens for \"%s\" "
+		"topX0=%d topY0=%d bottomX0=%d bottomY0=%d "
+		"topX1=%d topY1=%d bottomX1=%d bottomY1=%d \n",
+		local->name, priv->screenTopX[0], priv->screenTopY[0],
+		priv->screenBottomX[0], priv->screenBottomY[0],
+		priv->screenTopX[1], priv->screenTopY[1],
+		priv->screenBottomX[1], priv->screenBottomY[1]));
+}
+
+/*****************************************************************************
+ * xf86WcmInitialScreens
+ ****************************************************************************/
+
+void xf86WcmInitialScreens(LocalDevicePtr local)
+{
+	WacomDevicePtr priv = (WacomDevicePtr)local->private;
+	int i;
+
+	priv->tvoffsetX = 0;
+	priv->tvoffsetY = 0;
+	if (priv->twinview != TV_NONE)
+	{
+		xf86WcmInitialTVScreens(local);
+		return;
+	}
+
+	/* initial screen info */
+	priv->numScreen = screenInfo.numScreens;
+	priv->screenTopX[0] = 0;
+	priv->screenTopY[0] = 0;
+	priv->screenBottomX[0] = 0;
+	priv->screenBottomY[0] = 0;
+	for (i=0; i<screenInfo.numScreens; i++)
+	{
+#ifdef WCM_XORG
+		priv->screenTopX[i] = dixScreenOrigins[i].x;
+		priv->screenTopY[i] = dixScreenOrigins[i].y;
+		priv->screenBottomX[i] = dixScreenOrigins[i].x;
+		priv->screenBottomY[i] = dixScreenOrigins[i].y;
+
+		DBG(10, priv->debugLevel, ErrorF("xf86WcmInitialScreens from dix for \"%s\" "
+			"ScreenOrigins[%d].x=%d ScreenOrigins[%d].y=%d \n",
+			local->name, i, priv->screenTopX[i], i, priv->screenTopY[i]));
+#else
+		if (i > 0)
+		{
+			/* only support left to right in this case */
+			priv->screenTopX[i] = priv->screenBottomX[i-1];
+			priv->screenTopY[i] = 0;
+			priv->screenBottomX[i] = priv->screenTopX[i];
+			priv->screenBottomY[i] = 0;
+		}
+#endif
+		priv->screenBottomX[i] += screenInfo.screens[i]->width;
+		priv->screenBottomY[i] += screenInfo.screens[i]->height;
+
+		DBG(10, priv->debugLevel, ErrorF("xf86WcmInitialScreens for \"%s\" "
+			"topX[%d]=%d topY[%d]=%d bottomX[%d]=%d bottomY[%d]=%d \n",
+			local->name, i, priv->screenTopX[i], i, priv->screenTopY[i],
+			i, priv->screenBottomX[i], i, priv->screenBottomY[i]));
+	}
+}
