@@ -1,6 +1,6 @@
 /*
  * Copyright 1995-2002 by Frederic Lepied, France. <Lepied@XFree86.org>
- * Copyright 2002-2007 by Ping Cheng, Wacom. <pingc@wacom.com>
+ * Copyright 2002-2008 by Ping Cheng, Wacom. <pingc@wacom.com>
  *                                                                            
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -107,6 +107,8 @@ LocalDevicePtr xf86WcmAllocate(char* name, int flag)
 	priv->topY = 0;              /* Y top */
 	priv->bottomX = 0;           /* X bottom */
 	priv->bottomY = 0;           /* Y bottom */
+	priv->sizeX = 0;	     /* active X size */
+	priv->sizeY = 0;	     /* active Y size */
 	priv->factorX = 0.0;         /* X factor */
 	priv->factorY = 0.0;         /* Y factor */
 	priv->common = common;       /* common info pointer */
@@ -146,25 +148,31 @@ LocalDevicePtr xf86WcmAllocate(char* name, int flag)
 		for (j=0; j<256; j++)
 			priv->keys[i][j] = 0;
 
-	priv->nbuttons = MAX_BUTTONS;      /* Default number of buttons */
-	priv->relup = 5;		   /* Default relative wheel up event */
-	priv->reldn = 4;		   /* Default relative wheel down event */
-	priv->wheelup = 0;		   /* Default absolute wheel up event */
-	priv->wheeldn = 0;		   /* Default absolute wheel down event */
-	priv->striplup = 4;		   /* Default left strip up event */
-	priv->stripldn = 5;		   /* Default left strip down event */
-	priv->striprup = 4;		   /* Default right strip up event */
-	priv->striprdn = 5;		   /* Default right strip down event */
-	priv->naxes = 6;                   /* Default number of axes */
-	priv->debugLevel = 0;              /* debug level */
+	priv->nbuttons = MAX_BUTTONS;		/* Default number of buttons */
+	priv->relup = 5;			/* Default relative wheel up event */
+	priv->reldn = 4;			/* Default relative wheel down event */
+	
+	priv->wheelup = IsPad (priv) ? 5 : 0;	/* Default absolute wheel up event */
+	priv->wheeldn = IsPad (priv) ? 4 : 0;	/* Default absolute wheel down event */
+	priv->striplup = 4;			/* Default left strip up event */
+	priv->stripldn = 5;			/* Default left strip down event */
+	priv->striprup = 4;			/* Default right strip up event */
+	priv->striprdn = 5;			/* Default right strip down event */
+	priv->naxes = 6;			/* Default number of axes */
+	priv->debugLevel = 0;			/* debug level */
 	priv->numScreen = screenInfo.numScreens; /* configured screens count */
-	priv->currentScreen = 0;                 /* current screen in display */
+	priv->currentScreen = -1;                /* current screen in display */
 
+	priv->maxWidth = 0;			/* max active screen width */
+	priv->maxHeight = 0;			/* max active screen height */
 	priv->twinview = TV_NONE;		/* not using twinview gfx */
 	priv->tvoffsetX = 0;			/* none X edge offset for TwinView setup */
 	priv->tvoffsetY = 0;			/* none Y edge offset for TwinView setup */
 	for (i=0; i<4; i++)
 		priv->tvResolution[i] = 0;	/* unconfigured twinview resolution */
+	priv->wcmMMonitor = 1;		/* enabled (=1) to support multi-monitor desktop. */
+						/* disabled (=0) when user doesn't want to move the */
+						/* cursor from one screen to another screen */
 
 	/* JEJ - throttle sampling code */
 	priv->throttleValue = 0;
@@ -193,9 +201,6 @@ LocalDevicePtr xf86WcmAllocate(char* name, int flag)
 	common->wcmDevCls = &gWacomSerialDevice; /* device-specific functions */
 	common->wcmModel = NULL;                 /* model-specific functions */
 	common->wcmEraserID = 0;	/* eraser id associated with the stylus */
-	common->wcmMMonitor = 1;	/* enabled (=1) to support multi-monitor desktop. */
-					/* disabled (=0) when user doesn't want to move the */
-					/* cursor from one screen to another screen */
 	common->wcmTPCButtonDefault = 0; /* default Tablet PC button support is off */
 	common->wcmTPCButton = 
 		common->wcmTPCButtonDefault; /* set Tablet PC button on/off */
@@ -573,7 +578,7 @@ static LocalDevicePtr xf86WcmInit(InputDriverPtr drv, IDevPtr dev, int flags)
 		common->wcmFlags |= RAW_FILTERING_FLAG;
 	}
 
-#ifdef LINUX_INPUT
+#ifdef WCM_ENABLE_LINUXINPUT
 	if (xf86SetBoolOption(local->options, "USB",
 			(common->wcmDevCls == &gWacomUSBDevice)))
 	{
@@ -782,10 +787,10 @@ static LocalDevicePtr xf86WcmInit(InputDriverPtr drv, IDevPtr dev, int flags)
 	}
 
 	/* Cursor stays in one monitor in a multimonitor setup */
-	if ( !common->wcmMMonitor )
+	if ( !priv->wcmMMonitor )
 	{
-		common->wcmMMonitor = xf86SetBoolOption(local->options, "MMonitor", 1);
-		if ( !common->wcmMMonitor )
+		priv->wcmMMonitor = xf86SetBoolOption(local->options, "MMonitor", 1);
+		if ( !priv->wcmMMonitor )
 			xf86Msg(X_CONFIG, "%s: Cursor will stay in one monitor \n", common->wcmDevice);
 	}
 
