@@ -1,5 +1,5 @@
 /*
- * Copyright 2007 by Ping Cheng, Wacom Technology. <pingc@wacom.com>
+ * Copyright 2007-2008 by Ping Cheng, Wacom Technology. <pingc@wacom.com>
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -29,11 +29,12 @@
 #include "wcmFilter.h"
 
 extern void xf86WcmInitialCoordinates(LocalDevicePtr local, int axes);
-extern void xf86WcmInitialTVScreens(LocalDevicePtr local);
+extern void xf86WcmInitialScreens(LocalDevicePtr local);
 
 /*****************************************************************************
  * xf86WcmSetPadCoreMode
  ****************************************************************************/
+
 int xf86WcmSetPadCoreMode(LocalDevicePtr local)
 {
 	WacomDevicePtr priv = (WacomDevicePtr)local->private;
@@ -54,6 +55,88 @@ int xf86WcmSetPadCoreMode(LocalDevicePtr local)
 	return Success;
 }
 
+/*****************************************************************************
+* xf86WcmDevSwitchModeCall --
+*****************************************************************************/
+
+int xf86WcmDevSwitchModeCall(LocalDevicePtr local, int mode)
+{
+	WacomDevicePtr priv = (WacomDevicePtr)local->private;
+	int is_absolute = priv->flags & ABSOLUTE_FLAG;
+
+	DBG(3, priv->debugLevel, ErrorF("xf86WcmSwitchModeCall for %s to mode=%d\n", 
+		local->name, mode));
+
+	/* Pad is always in relative mode when it's a core device.
+	 * Always in absolute mode when it is not a core device.
+	 */
+	if (IsPad(priv))
+		return xf86WcmSetPadCoreMode(local);
+
+	if ((mode == Absolute) && !is_absolute)
+	{
+		priv->flags |= ABSOLUTE_FLAG;
+		xf86ReplaceStrOption(local->options, "Mode", "Absolute");
+		xf86WcmMappingFactor(local);
+		xf86WcmInitialCoordinates(local, 0);
+		xf86WcmInitialCoordinates(local, 1);
+	}
+	else if ((mode == Relative) && is_absolute)
+	{
+		priv->flags &= ~ABSOLUTE_FLAG; 
+		xf86ReplaceStrOption(local->options, "Mode", "Relative");
+		xf86WcmMappingFactor(local);
+		xf86WcmInitialCoordinates(local, 0);
+		xf86WcmInitialCoordinates(local, 1);
+	}
+	else if ( (mode != Absolute) && (mode != Relative))
+	{
+		DBG(10, priv->debugLevel, ErrorF("xf86WcmSwitchModeCall"
+			" for %s invalid mode=%d\n", local->name, mode));
+		return BadMatch;
+	}
+
+	return Success;
+}
+
+/*****************************************************************************
+* xf86WcmDevSwitchMode --
+*****************************************************************************/
+
+int xf86WcmDevSwitchMode(ClientPtr client, DeviceIntPtr dev, int mode)
+{
+	LocalDevicePtr local = (LocalDevicePtr)dev->public.devicePrivate;
+	WacomDevicePtr priv = (WacomDevicePtr)local->private;
+
+	DBG(3, priv->debugLevel, ErrorF("xf86WcmSwitchMode dev=%p mode=%d\n", 
+		(void *)dev, mode));
+
+	/* Share this call with sendAButton in wcmCommon.c */
+	return xf86WcmDevSwitchModeCall(local, mode);
+}
+
+/*****************************************************************************
+ * xf86WcmChangeScreen
+ ****************************************************************************/
+
+void xf86WcmChangeScreen(LocalDevicePtr local, int value)
+{
+	WacomDevicePtr priv = (WacomDevicePtr)local->private;
+
+	if (priv->screen_no != value)
+	{
+		priv->screen_no = value;
+		xf86ReplaceIntOption(local->options, "ScreenNo", value);
+	}
+
+	if (priv->screen_no != -1)
+		priv->currentScreen = priv->screen_no;
+	xf86WcmInitialScreens(local);
+	xf86WcmMappingFactor(local);
+	xf86WcmInitialCoordinates(local, 0);
+	xf86WcmInitialCoordinates(local, 1);
+}
+
 /*****************************************************************************
  * xf86WcmSetParam
  ****************************************************************************/
@@ -89,6 +172,7 @@ static int xf86WcmSetParam(LocalDevicePtr local, int param, int value)
 			/* Area definition is ok */
 			xf86ReplaceIntOption(local->options, "TopX", value);
 			priv->topX = xf86SetIntOption(local->options, "TopX", 0);
+			xf86WcmMappingFactor(local);
 			xf86WcmInitialCoordinates(local, 0);
 		}
 		break;
@@ -107,6 +191,7 @@ static int xf86WcmSetParam(LocalDevicePtr local, int param, int value)
 			/* Area definition is ok */
 			xf86ReplaceIntOption(local->options, "TopY", value);
 			priv->topY = xf86SetIntOption(local->options, "TopY", 0);
+			xf86WcmMappingFactor(local);
 			xf86WcmInitialCoordinates(local, 1);
 		}
 		break;
@@ -125,6 +210,7 @@ static int xf86WcmSetParam(LocalDevicePtr local, int param, int value)
 			/* Area definition is ok */
 			xf86ReplaceIntOption(local->options, "BottomX", value);
 			priv->bottomX = xf86SetIntOption(local->options, "BottomX", 0);
+			xf86WcmMappingFactor(local);
 			xf86WcmInitialCoordinates(local, 0);
 		}
 		break;
@@ -143,6 +229,7 @@ static int xf86WcmSetParam(LocalDevicePtr local, int param, int value)
 			/* Area definition is ok */
 			xf86ReplaceIntOption(local->options, "BottomY", value);
 			priv->bottomY = xf86SetIntOption(local->options, "BottomY", 0);
+			xf86WcmMappingFactor(local);
 			xf86WcmInitialCoordinates(local, 1);
 		}
 		break;
@@ -211,25 +298,9 @@ static int xf86WcmSetParam(LocalDevicePtr local, int param, int value)
 	    {
 		int is_absolute = priv->flags & ABSOLUTE_FLAG;
 		if ((value < 0) || (value > 1)) return BadValue;
-		if (value == is_absolute)
-			break;
-
-		if (IsPad(priv))
-			return xf86WcmSetPadCoreMode(local);
-
-		if (value) 
+		if (value != is_absolute)
 		{
-			priv->flags |= ABSOLUTE_FLAG;
-			xf86ReplaceStrOption(local->options, "Mode", "Absolute");
-			xf86WcmInitialCoordinates(local, 0);
-			xf86WcmInitialCoordinates(local, 1);
-		}
-		else
-		{
-			priv->flags &= ~ABSOLUTE_FLAG;
-			xf86ReplaceStrOption(local->options, "Mode", "Relative");
-			xf86WcmInitialCoordinates(local, 0);
-			xf86WcmInitialCoordinates(local, 1);
+			xf86WcmDevSwitchModeCall(local, value);
 		}
 		break;
 	    }
@@ -259,27 +330,29 @@ static int xf86WcmSetParam(LocalDevicePtr local, int param, int value)
 		xf86WcmSetParam (local, XWACOM_PARAM_BOTTOMY, common->wcmMaxY);
 		break;
 	    case XWACOM_PARAM_MMT:
-		if ((value != 0) && (value != 1)) return BadValue;
-		common->wcmMMonitor = value;
-		if (value)
+		if ((value != 0) && (value != 1)) 
+			return BadValue;
+		else if (priv->wcmMMonitor != value)
 		{
-			xf86ReplaceStrOption(local->options, "MMonitor", "on");
-		}
-		else
-		{
-			xf86ReplaceStrOption(local->options, "MMonitor", "off");
+			priv->wcmMMonitor = value;
+			if (value)
+				xf86ReplaceStrOption(local->options, "MMonitor", "on");
+			else
+				xf86ReplaceStrOption(local->options, "MMonitor", "off");
+			
+			xf86WcmMappingFactor(local);
 		}
 		break;
 	    case XWACOM_PARAM_TPCBUTTON:
-		if ((value != 0) && (value != 1)) return BadValue;
-		priv->common->wcmTPCButton = value;
-		if (value)
+		if ((value != 0) && (value != 1)) 
+			return BadValue;
+		else if (common->wcmTPCButton != value)
 		{
-			xf86ReplaceStrOption(local->options, "TPCButton", "on");
-		}
-		else
-		{
-			xf86ReplaceStrOption(local->options, "TPCButton", "off");
+			common->wcmTPCButton = value;
+			if (value)
+				xf86ReplaceStrOption(local->options, "TPCButton", "on");
+			else
+				xf86ReplaceStrOption(local->options, "TPCButton", "off");
 		}
 		break;
 	    case XWACOM_PARAM_CURSORPROX:
@@ -287,38 +360,19 @@ static int xf86WcmSetParam(LocalDevicePtr local, int param, int value)
 		{
 			if ((value > 255) || (value < 0))
 				return BadValue;
-			xf86ReplaceIntOption(local->options, "CursorProx",value);
-			common->wcmCursorProxoutDist = value;
+			else if (common->wcmCursorProxoutDist != value)
+			{
+				xf86ReplaceIntOption(local->options, "CursorProx",value);
+				common->wcmCursorProxoutDist = value;
+			}
 		}
 		break;
 	    case XWACOM_PARAM_SCREEN_NO:
-		if ( (priv->twinview == TV_NONE && 
-				screenInfo.numScreens == 1) 
-				|| value < -1 || 
-				(priv->twinview == TV_NONE && 
-				value >= screenInfo.numScreens) 
-				|| (priv->twinview != TV_NONE 
-				&& value > 1) )
+		if (value < -1 || value >= priv->numScreen) 
 			return BadValue;
 		else if (priv->screen_no != value)
 		{
-			priv->screen_no = value;
-			if (priv->twinview != TV_NONE)
-			{
-				if (priv->screen_no == -1)
-				{
-				    if (priv->twinview == TV_LEFT_RIGHT)
-					priv->tvoffsetX = 60;
-				    else
-					priv->tvoffsetY = 60;
-				}
-				else
-				{
-					priv->tvoffsetX = 0;
-					priv->tvoffsetY = 0;
-				}
-			}
-			xf86ReplaceIntOption(local->options, "ScreenNo", value);
+			xf86WcmChangeScreen(local, value);
 		}
 		break;
 	    case XWACOM_PARAM_TWINVIEW:
@@ -327,20 +381,7 @@ static int xf86WcmSetParam(LocalDevicePtr local, int param, int value)
 			if ((value > 2) || (value < 0) || screenInfo.numScreens != 1)
 				return BadValue;
 			priv->twinview = value;
-			switch(value) {
-			    case TV_NONE:
-				xf86ReplaceStrOption(local->options, "TwinView", "None");
-				break;
-			    case TV_ABOVE_BELOW:
-				xf86ReplaceStrOption(local->options, "TwinView", "Vertical");
-				break;
-			    case TV_LEFT_RIGHT:
-				xf86ReplaceStrOption(local->options, "TwinView", "Horizontal");
-				break;
-			    default:
-				return BadValue;
-			}
-			xf86WcmInitialTVScreens(local);
+			xf86WcmChangeScreen(local, priv->screen_no);
 		}
 		break;
 	    case XWACOM_PARAM_TVRESOLUTION0:
@@ -356,6 +397,7 @@ static int xf86WcmSetParam(LocalDevicePtr local, int param, int value)
 			{
 				rX = priv->tvResolution[0];
 				rY = priv->tvResolution[1];
+				sNum++;
 			}
 			else
 			{
@@ -377,6 +419,7 @@ static int xf86WcmSetParam(LocalDevicePtr local, int param, int value)
 				return BadValue;
 			priv->tvResolution[sNum++] = value & 0xffff;
 			priv->tvResolution[sNum] = (value >> 16) & 0xffff;
+			xf86WcmChangeScreen(local, priv->screen_no);
 			DBG(10, priv->debugLevel, ErrorF("xf86WcmSetParam " 
 					"to ResX=%d ResY=%d \n",
 				value & 0xffff, (value >> 16) & 0xffff));
@@ -487,6 +530,8 @@ static int xf86WcmSetParam(LocalDevicePtr local, int param, int value)
 			break;
 		    }
 
+		    xf86WcmInitialScreens(tmppriv->local);
+		    xf86WcmMappingFactor(tmppriv->local);
 		    xf86WcmInitialCoordinates(tmppriv->local, 0);
 		    xf86WcmInitialCoordinates(tmppriv->local, 1);
 
@@ -522,14 +567,18 @@ static int xf86WcmSetButtonParam(LocalDevicePtr local, int param, int value)
 			return BadValue;
 		else
 		{
-			if ((!(value & AC_TYPE)) && (value != priv->button[bn]))
+			if (((value & AC_TYPE) == AC_BUTTON) && (value != priv->button[bn]) && !number_keys )
 			{
+				/* assign button */
 				snprintf (st, sizeof (st), "Button%d", bn);
 				xf86ReplaceIntOption (local->options, st, value);
 				priv->button[bn] = xf86SetIntOption (local->options, st, bn); 
+			} 
+			else
+			{
+				setVal = &(priv->button [bn]);
+				keyP = priv->keys[bn];
 			}
-			setVal = &(priv->button [bn]);
-			keyP = priv->keys[bn];
 		}
 	}
 
@@ -568,6 +617,7 @@ static int xf86WcmSetButtonParam(LocalDevicePtr local, int param, int value)
 		keyP = priv->srdnk;
 		break;
 	}
+	/* assign keys */
 	if (keyP)
 	{
 		if (!number_keys)
@@ -575,8 +625,8 @@ static int xf86WcmSetButtonParam(LocalDevicePtr local, int param, int value)
 			*setVal = value;
 			number_keys = (value & AC_NUM_KEYS) >> 20;
 			DBG(10, priv->debugLevel, ErrorF(
-			"xf86WcmSetButtonParam value = %x number"
-			" of keys = %d\n", *setVal, number_keys));
+				"xf86WcmSetButtonParam value = %x number"
+				" of keys = %d\n", *setVal, number_keys));
 			if (number_keys)
 				keyP[button_keys++] = value & 0xffff;
 		}
@@ -586,6 +636,9 @@ static int xf86WcmSetButtonParam(LocalDevicePtr local, int param, int value)
 			{
 				keyP[button_keys++] = value & 0xffff;
 				keyP[button_keys++] = (value & 0xffff0000) >> 16;
+				DBG(10, priv->debugLevel, ErrorF(
+					"xf86WcmSetButtonParam got %d values = %x \n",
+					 button_keys, value));
 			}
 		}
 		if (button_keys >= number_keys)
@@ -709,10 +762,7 @@ static int xf86WcmGetParam(LocalDevicePtr local, int param)
 	    case XWACOM_PARAM_STOPX7:
 	    {
 		int sn = (param - XWACOM_PARAM_STOPX0) / 4; 
-		int numS = screenInfo.numScreens;
-		if (priv->twinview != TV_NONE)
-			numS = 2;
-		if (sn >= numS)
+		if (sn >= priv->numScreen)
 			return -1;
 		else
 			return priv->screenTopX[sn];
@@ -727,10 +777,7 @@ static int xf86WcmGetParam(LocalDevicePtr local, int param)
 	    case XWACOM_PARAM_STOPY7:
 	    {
 		int sn = (param - XWACOM_PARAM_STOPY0) / 4; 
-		int numS = screenInfo.numScreens;
-		if (priv->twinview != TV_NONE)
-			numS = 2;
-		if (sn >= numS)
+		if (sn >= priv->numScreen)
 			return -1;
 		else
 			return priv->screenTopY[sn];
@@ -745,10 +792,7 @@ static int xf86WcmGetParam(LocalDevicePtr local, int param)
 	    case XWACOM_PARAM_SBOTTOMX7:
 	    {
 		int sn = (param - XWACOM_PARAM_SBOTTOMX0) / 4; 
-		int numS = screenInfo.numScreens;
-		if (priv->twinview != TV_NONE)
-			numS = 2;
-		if (sn >= numS)
+		if (sn >= priv->numScreen)
 			return -1;
 		else
 			return priv->screenBottomX[sn];
@@ -763,10 +807,7 @@ static int xf86WcmGetParam(LocalDevicePtr local, int param)
 	    case XWACOM_PARAM_SBOTTOMY7:
 	    {
 		int sn = (param - XWACOM_PARAM_SBOTTOMY0) / 4; 
-		int numS = screenInfo.numScreens;
-		if (priv->twinview != TV_NONE)
-			numS = 2;
-		if (sn >= numS)
+		if (sn >= priv->numScreen)
 			return -1;
 		else
 			return priv->screenBottomY[sn];
@@ -806,7 +847,7 @@ static int xf86WcmGetParam(LocalDevicePtr local, int param)
 	    case XWACOM_PARAM_XYDEFAULT:
 		return -1;
 	    case XWACOM_PARAM_MMT:
-		return common->wcmMMonitor;
+		return priv->wcmMMonitor;
 	    case XWACOM_PARAM_TPCBUTTON:
 		return common->wcmTPCButton;
 	    case XWACOM_PARAM_CURSORPROX:
@@ -835,10 +876,7 @@ static int xf86WcmGetParam(LocalDevicePtr local, int param)
 	    case XWACOM_PARAM_SCREEN_NO:
 		return priv->screen_no;
 	    case XWACOM_PARAM_NUMSCREEN:
-		if (priv->twinview == TV_NONE)
-			return screenInfo.numScreens;
-		else
-			return 2;
+		return priv->numScreen;
 	}
 	DBG(10, priv->debugLevel, ErrorF("xf86WcmGetParam invalid param %d\n", param));
 	return -1;
@@ -851,10 +889,7 @@ static int xf86WcmGetParam(LocalDevicePtr local, int param)
 static int xf86WcmGetDefaultScreenInfo(LocalDevicePtr local, int param)
 {
 	WacomDevicePtr priv = (WacomDevicePtr)local->private;
-	int numS = screenInfo.numScreens, i = 0;
-
-	if (priv->twinview != TV_NONE)
-		numS = 2;
+	int numS = priv->numScreen;
 
 	switch (param)
 	{
@@ -870,17 +905,8 @@ static int xf86WcmGetDefaultScreenInfo(LocalDevicePtr local, int param)
 		int sn = (param - XWACOM_PARAM_STOPX0) / 4;
 		if (sn >= numS)
 			return -1;
-		else if (priv->twinview == TV_ABOVE_BELOW)
-			return 0;
-		else if (priv->twinview == TV_LEFT_RIGHT)
-			return (sn ? priv->tvResolution[0] : 0);
 		else
-		{
-			int x = 0;
-			for (i = 0; i<sn; i++)
-				x += screenInfo.screens[i]->width;
-			return x;
-		}
+			return priv->screenTopX[sn];
 	   }
 	case XWACOM_PARAM_STOPY0:
 	case XWACOM_PARAM_STOPY1:
@@ -894,17 +920,8 @@ static int xf86WcmGetDefaultScreenInfo(LocalDevicePtr local, int param)
 		int sn = (param - XWACOM_PARAM_STOPY0) / 4; 
 		if (sn >= numS)
 			return -1;
-		else if (priv->twinview == TV_ABOVE_BELOW)
-			return (sn ? priv->tvResolution[1] : 0);
-		else if (priv->twinview == TV_LEFT_RIGHT)
-			return 0;
 		else
-		{
-			int y = 0;
-			for (i = 0; i<sn; i++)
-				y += screenInfo.screens[i]->height;
-			return y;
-		}
+			return priv->screenTopY[sn];
 	   }
 	case XWACOM_PARAM_SBOTTOMX0:
 	case XWACOM_PARAM_SBOTTOMX1:
@@ -918,10 +935,8 @@ static int xf86WcmGetDefaultScreenInfo(LocalDevicePtr local, int param)
 		int sn = (param - XWACOM_PARAM_SBOTTOMX0) / 4; 
 		if (sn >= numS)
 			return -1;
-		else if (priv->twinview != TV_NONE)
-			return (sn ? priv->tvResolution[2] : priv->tvResolution[0]);
 		else
-			return screenInfo.screens[i]->width;
+			return priv->screenBottomX[sn];
 	   }
 	case XWACOM_PARAM_SBOTTOMY0:
 	case XWACOM_PARAM_SBOTTOMY1:
@@ -935,12 +950,11 @@ static int xf86WcmGetDefaultScreenInfo(LocalDevicePtr local, int param)
 		int sn = (param - XWACOM_PARAM_SBOTTOMY0) / 4; 
 		if (sn >= numS)
 			return -1;
-		else if (priv->twinview != TV_NONE)
-			return (sn ? priv->tvResolution[3] : priv->tvResolution[1]);
 		else
-			return screenInfo.screens[i]->height;
+			return priv->screenBottomY[sn];
 	   }
 	}
+	DBG(10, priv->debugLevel, ErrorF("xf86WcmGetDefaultScreenInfo invalid param %d\n", param));
 	return -1;
 }
 
@@ -957,6 +971,9 @@ static int xf86WcmGetDefaultParam(LocalDevicePtr local, int param)
 	if ( param >= XWACOM_PARAM_STOPX0 && param <= XWACOM_PARAM_SBOTTOMY2)
 		return xf86WcmGetDefaultScreenInfo(local, param);
 
+	if (param >= XWACOM_PARAM_BUTTON6 && param <= XWACOM_PARAM_BUTTON32)
+		return 0;
+
 	switch (param)
 	{
 	case XWACOM_PARAM_TOPX:
@@ -1016,49 +1033,7 @@ static int xf86WcmGetDefaultParam(LocalDevicePtr local, int param)
 	DBG(10, priv->debugLevel, ErrorF("xf86WcmGetDefaultParam invalid param %d\n",param));
 	return -1;
 }
-
-/*****************************************************************************
-* xf86WcmDevSwitchMode --
-*****************************************************************************/
-
-int xf86WcmDevSwitchMode(ClientPtr client, DeviceIntPtr dev, int mode)
-{
-	LocalDevicePtr local = (LocalDevicePtr)dev->public.devicePrivate;
-	WacomDevicePtr priv = (WacomDevicePtr)local->private;
-	int is_absolute = priv->flags & ABSOLUTE_FLAG;
-
-	DBG(3, priv->debugLevel, ErrorF("xf86WcmSwitchMode dev=%p mode=%d\n", (void *)dev, mode));
-
-	/* Pad is always in relative mode when it's a core device.
-	 * Always in absolute mode when it is not a core device.
-	 */
-	if (IsPad(priv))
-		return xf86WcmSetPadCoreMode(local);
-
-	if ((mode == Absolute) && !is_absolute)
-	{
-		priv->flags |= ABSOLUTE_FLAG;
-		xf86ReplaceStrOption(local->options, "Mode", "Absolute");
-		xf86WcmInitialCoordinates(local, 0);
-		xf86WcmInitialCoordinates(local, 1);
-	}
-	else if ((mode == Relative) && is_absolute)
-	{
-		priv->flags &= ~ABSOLUTE_FLAG; 
-		xf86ReplaceStrOption(local->options, "Mode", "Relative");
-		xf86WcmInitialCoordinates(local, 0);
-		xf86WcmInitialCoordinates(local, 1);
-	}
-	else if ( (mode != Absolute) && (mode != Relative))
-	{
-		DBG(10, priv->debugLevel, ErrorF("xf86WcmSwitchMode dev=%p invalid mode=%d\n",
-				(void *)dev, mode));
-		return BadMatch;
-	}
-
-	return Success;
-}
-
+ 
 /*****************************************************************************
  * xf86WcmDevChangeControl --
  ****************************************************************************/
@@ -1135,4 +1110,3 @@ int xf86WcmDevChangeControl(LocalDevicePtr local, xDeviceCtl* control)
 		r [i] = local->dev->valuator->axes [i].resolution;
         return rc;
 }
-
