@@ -1,6 +1,6 @@
 /*
  * Copyright 1995-2002 by Frederic Lepied, France. <Lepied@XFree86.org> 
- * Copyright 2002-2007 by Ping Cheng, Wacom Technology. <pingc@wacom.com>
+ * Copyright 2002-2008 by Ping Cheng, Wacom Technology. <pingc@wacom.com>
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -31,8 +31,11 @@
  * Frederic Lepied <lepied@xfree86.org>,
  * Brion Vibber <brion@pobox.com>,
  * Aaron Optimizer Digulla <digulla@hepe.com>,
- * Jonathan Layes <jonathan@layes.com>.
- * John Joganic <jej@j-arkadia.com>
+ * Jonathan Layes <jonathan@layes.com>,
+ * John Joganic <jej@j-arkadia.com>.
+ * 
+ * Support for hot plug-n-play by 
+ * Magnus Vigerlöf <Magnus.Vigerlof@ipbo.se>.
  */
 
 /*
@@ -58,9 +61,14 @@
  * 2007-06-25 47-pc0.7.8 - new release
  * 2007-10-25 47-pc0.7.9-1 - Support multimonitors in both horizonal and vertical settings
  * 2007-11-21 47-pc0.7.9-3 - Updated TwinView screen switch offset
+ * 2007-12-07 47-pc0.7.9-4 - Support Cintiq 12WX and Bamboo
+ * 2007-12-20 47-pc0.7.9-5 - multimonitor support update
+ * 2008-01-08 47-pc0.7.9-6 - Configure script change for Xorg 7.3 support
+ * 2008-01-17 47-pc0.7.9-7 - Preparing for hotplug-aware driver
+ * 2008-02-27 47-pc0.7.9-8 - Support Cintiq 20
  */
 
-static const char identification[] = "$Identification: 47-0.7.9-3 $";
+static const char identification[] = "$Identification: 47-0.7.9-8 $";
 
 /****************************************************************************/
 
@@ -80,6 +88,7 @@ extern Bool usbWcmInit(LocalDevicePtr pDev);
 extern int usbWcmGetRanges(LocalDevicePtr local);
 extern int xf86WcmDevChangeControl(LocalDevicePtr local, xDeviceCtl* control);
 extern int xf86WcmDevSwitchMode(ClientPtr client, DeviceIntPtr dev, int mode);
+extern void xf86WcmInitialScreens(LocalDevicePtr local);
 
 WacomModule gWacomModule =
 {
@@ -98,12 +107,15 @@ WacomModule gWacomModule =
 	xf86WcmDevReverseConvert,
 };
 
+static void xf86WcmKbdLedCallback(DeviceIntPtr di, LedCtrl * lcp)
+{
+}
+
 static int xf86WcmInitArea(LocalDevicePtr local)
 {
 	WacomDevicePtr priv = (WacomDevicePtr)local->private;
 	WacomToolAreaPtr area = priv->toolarea, inlist;
 	WacomCommonPtr common = priv->common;
-	int totalWidth = 0, maxHeight = 0;
 	double screenRatio, tabletRatio;
 
 	DBG(10, priv->debugLevel, ErrorF("xf86WcmInitArea\n"));
@@ -133,6 +145,9 @@ static int xf86WcmInitArea(LocalDevicePtr local)
 		area->bottomY = priv->bottomY = common->wcmMaxY;
 	}
 
+	if (priv->twinview != TV_NONE)
+		priv->numScreen = 2;
+
 	if (priv->screen_no != -1 &&
 		(priv->screen_no >= priv->numScreen || priv->screen_no < 0))
 	{
@@ -144,48 +159,16 @@ static int xf86WcmInitArea(LocalDevicePtr local)
 		}
 	}
 
-	/* Calculate the ratio according to KeepShape, TopX and TopY */
-	if (priv->screen_no != -1)
-	{
-		priv->currentScreen = priv->screen_no;
-		if (priv->twinview == TV_NONE)
-		{
-			totalWidth = screenInfo.screens[priv->currentScreen]->width;
-			maxHeight = screenInfo.screens[priv->currentScreen]->height;
-		}
-		else
-		{
-			totalWidth = priv->tvResolution[2*priv->currentScreen];
-			maxHeight = priv->tvResolution[2*priv->currentScreen+1];
-		}
-	}
-	else
-	{
-		int i;
-		for (i = 0; i < priv->numScreen; i++)
-		{
-			totalWidth += screenInfo.screens[i]->width;
-			if (maxHeight < screenInfo.screens[i]->height)
-				maxHeight=screenInfo.screens[i]->height;
-		}
-	}
-
-	if (priv->numScreen == 1)
-	{
-		priv->factorX = totalWidth
-			/ (double)(priv->bottomX - priv->topX - 2*priv->tvoffsetX);
-		priv->factorY = maxHeight
-			/ (double)(priv->bottomY - priv->topY - 2*priv->tvoffsetY);
-		DBG(2, priv->debugLevel, ErrorF("X factor = %.3g, Y factor = %.3g\n",
-			priv->factorX, priv->factorY));
-	}
+	/* need maxWidth and maxHeight for keepshape */
+	xf86WcmMappingFactor(local);
 
 	/* Maintain aspect ratio */
 	if (priv->flags & KEEP_SHAPE_FLAG)
 	{
-		screenRatio = totalWidth / (double)maxHeight;
-			tabletRatio = ((double)(common->wcmMaxX - priv->topX)) /
-				(common->wcmMaxY - priv->topY);
+
+		screenRatio = ((double)priv->maxWidth / (double)priv->maxHeight);
+		tabletRatio = ((double)(common->wcmMaxX - priv->topX) /
+				(double)(common->wcmMaxY - priv->topY));
 
 		DBG(2, priv->debugLevel, ErrorF("screenRatio = %.3g, "
 			"tabletRatio = %.3g\n", screenRatio, tabletRatio));
@@ -202,6 +185,8 @@ static int xf86WcmInitArea(LocalDevicePtr local)
 				screenRatio / tabletRatio + priv->topX;
 			area->bottomY = priv->bottomY = common->wcmMaxY;
 		}
+		/* active tablet size has been changed */
+		xf86WcmMappingFactor(local);
 	}
 	/* end keep shape */ 
 
@@ -258,6 +243,7 @@ void xf86WcmInitialCoordinates(LocalDevicePtr local, int axes)
 	WacomDevicePtr priv = (WacomDevicePtr)local->private;
 	WacomCommonPtr common = priv->common;
 	int tabletSize = 0, topx = 0, topy = 0;
+	int resolution;
 
 	/* x ax */
 	if ( !axes )
@@ -272,18 +258,20 @@ void xf86WcmInitialCoordinates(LocalDevicePtr local, int axes)
 				tabletSize = priv->bottomX - priv->topX;
 		}
 		if (priv->flags & ABSOLUTE_FLAG)
-			topx = priv->topX;
+			topx = priv->topX - priv->tvoffsetX;
 
-#if defined WCM_XORG && GET_ABI_MAJOR(ABI_XINPUT_VERSION) > 0
+		resolution = common->wcmResolX;
+#ifdef WCM_XORG_TABLET_SCALING
 		/* Ugly hack for Xorg 7.3, which doesn't call xf86WcmDevConvert
 		 * for coordinate conversion at the moment */
 		if (priv->flags & ABSOLUTE_FLAG) tabletSize -= topx;
 		topx = 0;
 		tabletSize = (int)((double)tabletSize * priv->factorX + 0.5);
+		resolution = (int)((double)resolution * priv->factorX + 0.5);
 #endif
 
 		InitValuatorAxisStruct(local->dev, 0, topx, tabletSize, 
-			common->wcmResolX, 0, common->wcmResolX); 
+			resolution, 0, resolution); 
 	}
 	else /* y ax */
 	{
@@ -297,127 +285,20 @@ void xf86WcmInitialCoordinates(LocalDevicePtr local, int axes)
 				tabletSize = priv->bottomY - priv->topY;
 		}
 		if (priv->flags & ABSOLUTE_FLAG)
-			topy = priv->topY;
+			topy = priv->topY - priv->tvoffsetY;
 
-#if defined WCM_XORG && GET_ABI_MAJOR(ABI_XINPUT_VERSION) > 0
+		resolution = common->wcmResolY;
+#ifdef WCM_XORG_TABLET_SCALING
 		/* Ugly hack for Xorg 7.3, which doesn't call xf86WcmDevConvert
 		 * for coordinate conversion at the moment */
 		if (priv->flags & ABSOLUTE_FLAG) tabletSize -= topy;
 		topy = 0;
 		tabletSize = (int)((double)tabletSize * priv->factorY + 0.5);
+		resolution = (int)((double)resolution * priv->factorY + 0.5);
 #endif
 
 		InitValuatorAxisStruct(local->dev, 1, topy, tabletSize, 
-			common->wcmResolY, 0, common->wcmResolY); 
-	}
-}
-
-/*****************************************************************************
- * xf86WcmInitialTVScreens
- ****************************************************************************/
-
-void xf86WcmInitialTVScreens(LocalDevicePtr local)
-{
-	WacomDevicePtr priv = (WacomDevicePtr)local->private;
-
-	priv->tvoffsetX = 0;
-	priv->tvoffsetY = 0;
-	if (priv->twinview == TV_NONE)
-		return;
-
-	if (priv->twinview == TV_LEFT_RIGHT)
-	{
-		if (priv->screen_no == -1)
-			priv->tvoffsetX = 60;
-
-		/* default resolution */
-		if(!priv->tvResolution[0])
-		{
-			priv->tvResolution[0] = screenInfo.screens[0]->width/2;
-			priv->tvResolution[1] = screenInfo.screens[0]->height;
-			priv->tvResolution[2] = priv->tvResolution[0];
-			priv->tvResolution[3] = priv->tvResolution[1];
-		}
-	}
-	else if (priv->twinview == TV_ABOVE_BELOW)
-	{
-		if (priv->screen_no == -1)
-			priv->tvoffsetY = 60;
-
-		if(!priv->tvResolution[0])
-		{
-			priv->tvResolution[0] = screenInfo.screens[0]->width;
-			priv->tvResolution[1] = screenInfo.screens[0]->height/2;
-			priv->tvResolution[2] = priv->tvResolution[0];
-			priv->tvResolution[3] = priv->tvResolution[1];
-		}
-	}
-
-	/* initial screen info */
-	priv->screenTopX[0] = 0;
-	priv->screenTopY[0] = 0;
-	priv->screenBottomX[0] = priv->tvResolution[0];
-	priv->screenBottomY[0] = priv->tvResolution[1];
-	if (priv->twinview == TV_ABOVE_BELOW)
-	{
-		priv->screenTopX[1] = 0;
-		priv->screenTopY[1] = priv->tvResolution[1];
-		priv->screenBottomX[1] = priv->tvResolution[2];
-		priv->screenBottomY[1] = priv->tvResolution[1] + priv->tvResolution[3];
-	}
-	if (priv->twinview == TV_LEFT_RIGHT)
-	{
-		priv->screenTopX[1] = priv->tvResolution[0];
-		priv->screenTopY[1] = 0;
-		priv->screenBottomX[1] = priv->tvResolution[0] + priv->tvResolution[2];
-		priv->screenBottomY[1] = priv->tvResolution[3];
-	}
-
-	DBG(10, priv->debugLevel, ErrorF("xf86WcmInitialTVScreens for \"%s\" "
-		"topX0=%d topY0=%d bottomX0=%d bottomY0=%d "
-		"topX1=%d topY1=%d bottomX1=%d bottomY1=%d \n",
-		local->name, priv->screenTopX[0], priv->screenTopY[0],
-		priv->screenBottomX[0], priv->screenBottomY[0],
-		priv->screenTopX[1], priv->screenTopY[1],
-		priv->screenBottomX[1], priv->screenBottomY[1]));
-}
-
-/*****************************************************************************
- * xf86WcmInitialScreens
- ****************************************************************************/
-
-void xf86WcmInitialScreens(LocalDevicePtr local)
-{
-	WacomDevicePtr priv = (WacomDevicePtr)local->private;
-	int i;
-
-	if (priv->twinview != TV_NONE)
-		return;
-
-	/* initial screen info */
-	priv->screenTopX[0] = 0;
-	priv->screenTopY[0] = 0;
-	priv->screenBottomX[0] = 0;
-	priv->screenBottomY[0] = 0;
-	for (i=0; i<screenInfo.numScreens; i++)
-	{
-#if !(defined WCM_XFREE86 || GET_ABI_MAJOR(ABI_XINPUT_VERSION) == 0)
-		priv->screenTopX[i] = dixScreenOrigins[i].x;
-		priv->screenTopY[i] = dixScreenOrigins[i].y;
-		priv->screenBottomX[i] = dixScreenOrigins[i].x;
-		priv->screenBottomY[i] = dixScreenOrigins[i].y;
-#endif
-		priv->screenBottomX[i] += screenInfo.screens[i]->width;
-		priv->screenBottomY[i] += screenInfo.screens[i]->height;
-
-#if defined WCM_XFREE86 || GET_ABI_MAJOR(ABI_XINPUT_VERSION) == 0
-		priv->screenTopX[i] = priv->screenBottomX[i];
-		priv->screenTopY[i] = priv->screenBottomY[i];
-#endif
-		DBG(10, priv->debugLevel, ErrorF("xf86WcmInitialScreens for \"%s\" "
-			"topX[%d]=%d topY[%d]=%d bottomX[%d]=%d bottomY[%d]=%d \n",
-			local->name, i, priv->screenTopX[i], i, priv->screenTopY[i],
-			i, priv->screenBottomX[i], i, priv->screenBottomY[i]));
+			resolution, 0, resolution); 
 	}
 }
 
@@ -430,7 +311,7 @@ static int xf86WcmRegisterX11Devices (LocalDevicePtr local)
 {
 	WacomDevicePtr priv = (WacomDevicePtr)local->private;
 	WacomCommonPtr common = priv->common;
-	CARD8 butmap[MAX_BUTTONS];
+	CARD8 butmap[MAX_BUTTONS+1];
 	int nbaxes, nbbuttons, nbkeys;
 	int loop;
 
@@ -450,6 +331,9 @@ static int xf86WcmRegisterX11Devices (LocalDevicePtr local)
 		IsPad(priv) ? "pad" : "eraser",
 		nbbuttons, nbkeys, nbaxes));
 
+	/* initialize screen bounding rect */
+	xf86WcmInitialScreens(local);
+
 	if (xf86WcmInitArea(local) == FALSE)
 	{
 		return FALSE;
@@ -503,41 +387,50 @@ static int xf86WcmRegisterX11Devices (LocalDevicePtr local)
 	}
 
 
-	if (nbkeys)
+	/* only initial KeyClass and LedFeedbackClass once */
+	if (!priv->wcmInitKeyClassCount)
 	{
-		KeySymsRec wacom_keysyms;
-		KeySym keymap[256];
-
-		for (loop = 0; loop < nbkeys; loop++)
-			if ((priv->button [loop] & AC_TYPE) == AC_KEY)
-				keymap [loop] = priv->button [loop] & AC_CODE;
-			else
-				keymap [loop] = NoSymbol;
-		for(loop = nbkeys; loop<256; loop++)
-			keymap [loop] = NoSymbol;
-
-		/* There seems to be a long-standing misunderstanding about
-		 * how a keymap should be defined. All tablet drivers from
-		 * stock X11 source tree are doing it wrong: they leave first
-		 * 8 keysyms as VoidSymbol's, and are passing 8 as minimum
-		 * key code. But if you look at SetKeySymsMap() from
-		 * programs/Xserver/dix/devices.c you will see that
-		 * Xserver does not require first 8 keysyms; it supposes
-		 * that the map begins at minKeyCode.
-		 *
-		 * It could be that this assumption is a leftover from
-		 * earlier XFree86 versions, but that's out of our scope.
-		 * This also means that no keys on extended input devices
-		 * with their own keycodes (e.g. tablets) were EVER used.
-		 */
-		wacom_keysyms.map = keymap;
-		/* minKeyCode = 8 because this is the min legal key code */
-		wacom_keysyms.minKeyCode = 8;
-		wacom_keysyms.maxKeyCode = 255;
-		wacom_keysyms.mapWidth = 1;
-		if (InitKeyClassDeviceStruct(local->dev, &wacom_keysyms, NULL) == FALSE)
+		if (nbkeys)
 		{
-			ErrorF("unable to init key class device\n");
+			KeySymsRec wacom_keysyms;
+			KeySym keymap[256];
+
+			for (loop = 0; loop < nbkeys; loop++)
+				if ((priv->button [loop] & AC_TYPE) == AC_KEY)
+					keymap [loop] = priv->button [loop] & AC_CODE;
+				else
+					keymap [loop] = NoSymbol;
+			for(loop = nbkeys; loop<256; loop++)
+				keymap [loop] = NoSymbol;
+
+			/* There seems to be a long-standing misunderstanding about
+			 * how a keymap should be defined. All tablet drivers from
+			 * stock X11 source tree are doing it wrong: they leave first
+			 * 8 keysyms as VoidSymbol's, and are passing 8 as minimum
+			 * key code. But if you look at SetKeySymsMap() from
+			 * programs/Xserver/dix/devices.c you will see that
+			 * Xserver does not require first 8 keysyms; it supposes
+			 * that the map begins at minKeyCode.
+			 *
+			 * It could be that this assumption is a leftover from
+			 * earlier XFree86 versions, but that's out of our scope.
+			 * This also means that no keys on extended input devices
+			 * with their own keycodes (e.g. tablets) were EVER used.
+			 */
+			wacom_keysyms.map = keymap;
+			/* minKeyCode = 8 because this is the min legal key code */
+			wacom_keysyms.minKeyCode = 8;
+			wacom_keysyms.maxKeyCode = 255;
+			wacom_keysyms.mapWidth = 1;
+			if (InitKeyClassDeviceStruct(local->dev, &wacom_keysyms, NULL) == FALSE)
+			{
+				ErrorF("unable to init key class device\n");
+				return FALSE;
+			}
+		}
+
+		if(InitLedFeedbackClassDeviceStruct (local->dev, xf86WcmKbdLedCallback) == FALSE) {
+			ErrorF("unable to init led feedback device struct\n");
 			return FALSE;
 		}
 	}
@@ -547,12 +440,6 @@ static int xf86WcmRegisterX11Devices (LocalDevicePtr local)
 	xf86MotionHistoryAllocate(local);
 #endif
 
-	/* initialize screen bounding rect */
-	if (priv->twinview != TV_NONE)
-		xf86WcmInitialTVScreens(local);
-	else
-		xf86WcmInitialScreens(local);
-
 	/* x */
 	xf86WcmInitialCoordinates(local, 0);
 
@@ -811,11 +698,13 @@ static int xf86WcmDevProc(DeviceIntPtr pWcm, int what)
 		 */
 		case DEVICE_INIT:
 			priv->wcmDevOpenCount = 0;
+			priv->wcmInitKeyClassCount = 0;
 			if (!xf86WcmDevOpen(pWcm))
 			{
 				DBG(1, priv->debugLevel, ErrorF("xf86WcmProc INIT FAILED\n"));
 				return !Success;
 			}
+			priv->wcmInitKeyClassCount++;
 			priv->wcmDevOpenCount++;
 			break; 
 
@@ -854,7 +743,8 @@ static int xf86WcmDevProc(DeviceIntPtr pWcm, int what)
 /*****************************************************************************
  * xf86WcmDevConvert --
  *  Convert X & Y valuators so core events can be generated with 
- *  coordinates that are scaled and suitable for screen resolution. ****************************************************************************/
+ *  coordinates that are scaled and suitable for screen resolution.
+ ****************************************************************************/
 
 static Bool xf86WcmDevConvert(LocalDevicePtr local, int first, int num,
 		int v0, int v1, int v2, int v3, int v4, int v5, int* x, int* y)
@@ -862,7 +752,8 @@ static Bool xf86WcmDevConvert(LocalDevicePtr local, int first, int num,
 	WacomDevicePtr priv = (WacomDevicePtr) local->private;
 	double temp;
     
-	DBG(6, priv->debugLevel, ErrorF("xf86WcmDevConvert v0=%d v1=%d \n", v0, v1));
+	DBG(6, priv->debugLevel, ErrorF("xf86WcmDevConvert v0=%d v1=%d on screen %d \n",
+		 v0, v1, priv->currentScreen));
 
 	if (first != 0 || num == 1) 
  		return FALSE;
@@ -872,94 +763,26 @@ static Bool xf86WcmDevConvert(LocalDevicePtr local, int first, int num,
 
 	if (priv->flags & ABSOLUTE_FLAG)
 	{
+		int leftPadding = 0;
+		int topPadding = 0;				
+
+		v0 = v0 - priv->topX - priv->tvoffsetX;
+		v1 = v1 - priv->topY - priv->tvoffsetY;
+
 		if (priv->twinview == TV_NONE)
 		{
-			v0 = v0 > priv->bottomX ? priv->bottomX - priv->topX : 
-				v0 < priv->topX ? 0 : v0 - priv->topX;
-			v1 = v1 > priv->bottomY ? priv->bottomY - priv->topY : 
-				v1 < priv->topY ? 0 : v1 - priv->topY;
-
-			if (priv->common->wcmMMonitor)
+			if (priv->screen_no == -1)
 			{
-				int i, totalWidth, leftPadding = 0;
-				if (priv->screen_no == -1)
-				{
-					for (i = 0; i < priv->currentScreen; i++)
-						leftPadding += screenInfo.screens[i]->width;
-					for (totalWidth = leftPadding; i < priv->numScreen; i++)
-						totalWidth += screenInfo.screens[i]->width;
-				}
-				else
-				{
-					leftPadding = 0;
-					totalWidth = screenInfo.screens[priv->currentScreen]->width;
-				}
-				v0 -= (priv->bottomX - priv->topX) * leftPadding
-					/ (double)totalWidth + 0.5;
+				leftPadding = priv->screenTopX[priv->currentScreen];
+				topPadding = priv->screenTopY[priv->currentScreen];				
 			}
+			*x = - leftPadding;
+			*y = - topPadding;
 		}
 		else
 		{
-			v0 -= priv->topX - priv->tvoffsetX;
-			v1 -= priv->topY - priv->tvoffsetY;
-			if (priv->twinview == TV_LEFT_RIGHT)
-			{
-				if (v0 > priv->bottomX - priv->tvoffsetX && priv->screen_no == -1)
-				{
-					if (priv->currentScreen == 0)
-						v0 = priv->bottomX - priv->tvoffsetX;
-					else
-					{
-						v0 -= priv->bottomX - priv->topX - 2*priv->tvoffsetX;
-						if (v0 > priv->bottomX - priv->tvoffsetX)
-							v0 = 2*(priv->bottomX - priv->tvoffsetX) - v0;
-					}
-				}
-				if (priv->currentScreen == 1)
-				{
-					*x = priv->tvResolution[0] + priv->tvResolution[2]
-						* v0 / (priv->bottomX - priv->topX - 2*priv->tvoffsetX);
-					*y = v1 * priv->tvResolution[3] /
-						(priv->bottomY - priv->topY - 2*priv->tvoffsetY) + 0.5;
-				}
-				else
-				{
-					*x = priv->tvResolution[0] * v0 
-						 / (priv->bottomX - priv->topX - 2*priv->tvoffsetX);
-					*y = v1 * priv->tvResolution[1] /
-						(priv->bottomY - priv->topY - 2*priv->tvoffsetY) + 0.5;
-				}
-			}
-			if (priv->twinview == TV_ABOVE_BELOW)
-			{
-				if (v1 > priv->bottomY - priv->tvoffsetY && priv->screen_no == -1)
-				{
-					if (priv->currentScreen == 0)
-						v1 = priv->bottomY - priv->tvoffsetY;
-					else
-					{
-						v1 -= priv->bottomY - priv->topY - 2*priv->tvoffsetY;
-						if (v1 > priv->bottomY - priv->tvoffsetY)
-							v1 = 2*(priv->bottomY - priv->tvoffsetY) - v1;
-					}
-				}
-				if (priv->currentScreen == 1)
-				{
-					*x = v0 * priv->tvResolution[2] /
-						(priv->bottomX - priv->topX - 2*priv->tvoffsetX) + 0.5;
-					*y = priv->tvResolution[1] + 
-						priv->tvResolution[3] * v1 / 
-						(priv->bottomY - priv->topY - 2*priv->tvoffsetY);
-				}
-				else
-				{
-					*x = v0 * priv->tvResolution[0] /
-						(priv->bottomX - priv->topX - 2*priv->tvoffsetX) + 0.5;
-					*y = priv->tvResolution[1] * v1 /
-						(priv->bottomY - priv->topY - 2*priv->tvoffsetY);
-				}
-			}
-			return TRUE;
+			*x = priv->screenTopX[priv->currentScreen];
+			*y = priv->screenTopY[priv->currentScreen];
 		}
 	}
 	temp = ((double)v0 * priv->factorX + 0.5);
@@ -967,7 +790,30 @@ static Bool xf86WcmDevConvert(LocalDevicePtr local, int first, int num,
 	temp = ((double)v1 * priv->factorY + 0.5);
 	*y += temp;
 
-	DBG(6, priv->debugLevel, ErrorF("Wacom converted v0=%d v1=%d to x=%d y=%d\n", v0, v1, *x, *y));
+	DBG(6, priv->debugLevel, ErrorF("xf86WcmDevConvert v0=%d v1=%d to x=%d y=%d\n", v0, v1, *x, *y));
+	if ((priv->screen_no != -1 || !priv->wcmMMonitor) && (priv->flags & ABSOLUTE_FLAG))
+	{
+		DBG(6, priv->debugLevel, ErrorF("xf86WcmDevConvert restricted (%d,%d)", *x, *y));
+		if (priv->twinview == TV_NONE)
+		{
+			if (*x < 1) *x = 0;
+			if (*y < 1) *y = 0;
+			if (*x >= priv->screenBottomX[priv->currentScreen] - priv->screenTopX[priv->currentScreen])
+				*x = priv->screenBottomX[priv->currentScreen] - priv->screenTopX[priv->currentScreen]-1;
+			if (*y >= priv->screenBottomY[priv->currentScreen] - priv->screenTopY[priv->currentScreen])
+				*y = priv->screenBottomY[priv->currentScreen] - priv->screenTopY[priv->currentScreen]-1;
+		}
+		else
+		{
+			if (*x < priv->screenTopX[priv->currentScreen]+1) *x = priv->screenTopX[priv->currentScreen];
+			if (*y < priv->screenTopY[priv->currentScreen]+1) *y = priv->screenTopY[priv->currentScreen];
+			if (*x >= priv->screenBottomX[priv->currentScreen])
+				*x = priv->screenBottomX[priv->currentScreen]-1;
+			if (*y >= priv->screenBottomY[priv->currentScreen])
+				*y = priv->screenBottomY[priv->currentScreen]-1;
+		}
+		DBG(6, priv->debugLevel, ErrorF(" to x=%d y=%d\n", *x, *y));
+	}
 	return TRUE;
 }
 
@@ -975,7 +821,8 @@ static Bool xf86WcmDevConvert(LocalDevicePtr local, int first, int num,
  * xf86WcmDevReverseConvert --
  *  Convert X and Y to valuators in relative mode where the position of 
  *  the core pointer must be translated into device cootdinates before 
- *  the extension and core events are generated in Xserver. ****************************************************************************/
+ *  the extension and core events are generated in Xserver.
+ ****************************************************************************/
 
 static Bool xf86WcmDevReverseConvert(LocalDevicePtr local, int x, int y,
 		int* valuators)
